<?php

namespace Andrew\ModeratorPanel\Pub\Controller;
use XF\Mvc\ParameterBag;

class UserNote extends \XF\Pub\Controller\AbstractController
{
    public function actionSave(ParameterBag $params)
    {
        $visitor = \XF::visitor();

        if (!$visitor->canAddUserNote())
        {
            return $this->noPermission();
        }

        $noteId = $params->note_id;
        $withData = $this->filter('_xfWithData', 'bool');
        $inlineEdit = $this->filter('_xfInlineEdit', 'bool');
        $silent = $this->filter('silent', 'bool');
        $clear_edit = $this->filter('clear_edit', 'bool');

        $user_id = $this->filter('user_id', 'int');
        $message = $this->plugin('XF:Editor')->fromInput('message');

        if($inlineEdit)
        {
            $repo = $this->repository('Andrew\ModeratorPanel:UserNote');
            $finder = $repo->findUserNote();
            $user_note = $finder->where('note_id', $noteId)->fetchOne();

            if($silent)
            {
                if($clear_edit)
                {
                    $user_note->edit_user_id = 0;
                }
            }
            else
            {
                $user_note->last_edit_date = \XF::$time;
                $user_note->edit_user_id = $visitor->user_id;
            }

        }
        else
        {
            $user_note = \XF::em()->create('Andrew\ModeratorPanel:UserNote');
            $user_note->note_user_id = $user_id;
            $user_note->username = $visitor->username;
            $user_note->user_id = $visitor->user_id;
            $user_note->last_edit_date = 0;
        }

        $user_note->message = $message;
        $user_note->save();

        $finder = \XF::finder('XF:User');
        $user = $finder
            ->where('user_id',$user_id)
            ->fetchOne();

        if ($withData && $inlineEdit)
        {
            $viewParams = [
                'user_note' => $user_note,
            ];

            $reply = $this->view('Andrew\ModeratorPanel:UserNote\Edit', 'andrew_moderatorpanel_user_note_edit_saved', $viewParams);
            $reply->setJsonParam('message', \XF::phrase('your_changes_have_been_saved'));
            return $reply;
        }
        else
        {
            return $this->redirect($this->buildLink('moderatorpanel/user/', $user) . '#user_notes');
        }
    }
    public function actionEdit(ParameterBag $params)
    {
        $visitor = \XF::visitor();

        $note_id = $params->note_id;

        $repo = $this->repository('Andrew\ModeratorPanel:UserNote');
        $finder = $repo->findUserNote();
        $user_note = $finder->where('note_id', $note_id)->fetchOne();

        if (!$visitor->canEditOwnUserNote() and $visitor->user_id == $user_note->user_id)
        {
            return $this->noPermission();
        }

        if (!$visitor->canEditOtherUserNote() and $visitor->user_id != $user_note->user_id)
        {
            return $this->noPermission();
        }

        $viewParams = [
            'user_note' => $user_note,

            'quickEdit' => $this->filter('_xfWithData', 'bool')
        ];
        return $this->view('Andrew\ModeratorPanel:UserNote\Edit', 'andrew_moderatorpanel_user_note_edit', $viewParams);

    }
    public function actionDelete(ParameterBag $params)
    {
        $visitor = \XF::visitor();
        $note_id = $params->note_id;

        $repo = $this->repository('Andrew\ModeratorPanel:UserNote');
        $finder = $repo->findUserNote();
        $user_note = $finder->where('note_id', $note_id)->fetchOne();

        if (!$visitor->canDeleteOwnUserNote() and $visitor->user_id == $user_note->user_id)
        {
            return $this->noPermission();
        }

        if (!$visitor->canDeleteOtherUserNote() and $visitor->user_id != $user_note->user_id)
        {
            return $this->noPermission();
        }

        $user_id = $user_note->note_user_id;

        $user_note->delete();


        if($user_id)
        {
            $finder = \XF::finder('XF:User');
            $user = $finder
                ->where('user_id',$user_id)
                ->fetchOne();

            return $this->redirect($this->buildLink('moderatorpanel/user/#user_notes',$user));
        }
        else
        {
            return $this->redirect($this->buildLink('moderatorpanel/'));
        }

    }
}