<?php

namespace Andrew\ModeratorPanel;

use XF\AddOn\AbstractSetup;
use XF\AddOn\StepRunnerInstallTrait;
use XF\AddOn\StepRunnerUninstallTrait;
use XF\AddOn\StepRunnerUpgradeTrait;

use XF\Db\Schema\Create;
use XF\Db\Schema\Alter;

class Setup extends AbstractSetup
{
	use StepRunnerInstallTrait;
	use StepRunnerUpgradeTrait;
	use StepRunnerUninstallTrait;


    public function installStep1()
    {
        $sm = $this->schemaManager();
        $sm->createTable('xf_andrew_mp_user_note', function(Create $table)
        {
            $table->addColumn('note_id', 'int')->autoIncrement();
            $table->addColumn('note_user_id', 'int')->nullable(true)->setDefault(null);
            $table->addColumn('user_id', 'int')->nullable(true)->setDefault(null);
            $table->addColumn('username','varchar', 250)->nullable(true)->setDefault(null);
            $table->addColumn('create_date', 'int',10)->nullable(true)->setDefault(\XF::$time);
            $table->addColumn('message', 'mediumtext')->nullable(true)->setDefault(null);
            $table->addColumn('edit_user_id', 'int')->nullable(true)->setDefault(null);
            $table->addColumn('last_edit_date', 'int',10)->setDefault(0);
            $table->addPrimaryKey('note_id');
        });
    }

    public function upgrade1020270Step1()
    {

            $sm = $this->schemaManager();
            $sm->CreateTable('xf_andrew_mp_user_note', function (Create $table)
            {

                $table->addColumn('note_id', 'int')->autoIncrement();
                $table->addColumn('note_user_id', 'int')->nullable(true)->setDefault(null);
                $table->addColumn('user_id', 'int')->nullable(true)->setDefault(null);
                $table->addColumn('username','varchar', 250)->nullable(true)->setDefault(null);
                $table->addColumn('create_date', 'int',10)->nullable(true)->setDefault(\XF::$time);
                $table->addColumn('message', 'mediumtext')->nullable(true)->setDefault(null);
                $table->addPrimaryKey('note_id');

            });
    }

    public function installStep2()
    {
        $sm = $this->schemaManager();
        $sm->alterTable('xf_user', function(Alter $table)
        {
            $table->addColumn('andrew_user_note_count', 'int')->setDefault(0);
            $table->addColumn('andrew_reg_country','varchar', 250)->nullable(true)->setDefault(null);
        });
    }

    public function installStep3()
    {
        $sm = $this->schemaManager();
        $sm->alterTable('xf_warning', function(Alter $table)
        {
            $table->addColumn('andrew_mp_thread_id', 'int')->setDefault(0);
        });
    }

    public function installStep4()
    {
        $sm = $this->schemaManager();
        $sm->alterTable('xf_user_ignored', function(Alter $table)
        {
            $table->addColumn('andrew_forced', 'int')->setDefault(0);
            $table->addColumn('andrew_forced_user_id', 'int')->setDefault(0);
            $table->addColumn('andrew_forced_datetime', 'int')->setDefault(\XF::$time);
        });
    }

    public function upgrade1050070Step1()
    {
        $sm = $this->schemaManager();
        $sm->alterTable('xf_user', function(Alter $table)
        {
            $table->addColumn('andrew_user_note_count', 'int')->setDefault(0);
        });
    }
    
    public function upgrade1060070Step1()
    {
        $sm = $this->schemaManager();
        $sm->alterTable('xf_user_ignored', function(Alter $table)
        {
            $table->addColumn('andrew_forced', 'int')->setDefault(0);
            $table->addColumn('andrew_forced_user_id', 'int')->setDefault(0);
            $table->addColumn('andrew_forced_datetime', 'int')->setDefault(\XF::$time);
        });
    }
    public function upgrade1060270Step1()
    {
        $sm = $this->schemaManager();
        $sm->alterTable('xf_warning', function(Alter $table)
        {
            $table->addColumn('andrew_mp_thread_id', 'int')->setDefault(0);
        });
        $sm->alterTable('xf_user', function(Alter $table)
        {
            $table->dropColumns('andrew_mp_thread_id');
        });

    }
    public function upgrade1070070Step1()
    {
        $sm = $this->schemaManager();
        $sm->alterTable('xf_andrew_mp_user_note', function(Alter $table)
        {
            $table->addColumn('edit_user_id', 'int')->nullable(true)->setDefault(null);
            $table->addColumn('last_edit_date', 'int',10)->nullable(true)->setDefault(\XF::$time);
        });
    }
    public function upgrade1090070Step1()
    {
        $sm = $this->schemaManager();
        $sm->alterTable('xf_user', function(Alter $table)
        {
            $table->addColumn('andrew_reg_country','varchar', 250)->nullable(true)->setDefault(null);
        });
    }
    public function uninstallStep1()
    {
        $sm = $this->schemaManager();
        $sm->dropTable('xf_andrew_mp_user_note');
    }

    public function uninstallStep2()
    {
        $sm = $this->schemaManager();
        $sm->alterTable('xf_user', function(Alter $table)
        {
            $table->dropColumns('andrew_user_note_count');
            $table->dropColumns('andrew_reg_country');
        });
    }
    public function uninstallStep3()
    {
        $sm = $this->schemaManager();
        $sm->alterTable('xf_warning', function(Alter $table)
        {
            $table->dropColumns('andrew_mp_thread_id');
        });
    }
    public function uninstallStep4()
    {
        $sm = $this->schemaManager();
        $sm->alterTable('xf_user_ignored', function(Alter $table)
        {
            $table->dropColumns('andrew_forced');
            $table->dropColumns('andrew_forced_user_id');
            $table->dropColumns('andrew_forced_datetime');
        });
    }
}